import java.awt.*;
import java.awt.event.*;
import java.io.*;

class Main extends Frame {
    int cellSize = 125;

    Main(File dir) {
        String[] filenames;

        if (dir.isDirectory()) {
            // If directory, get files.
            filenames = dir.list();
        } else {
            // Otherwise, only show one image.
            filenames = new String[1];
            filenames[0] = dir.getName();

            // Set dir to be the directory rather than the file.
            dir = new File(dir.getParent());
        }
    
        for (int i=0; i<filenames.length; i++) {
            if (filenames[i].toLowerCase().endsWith(".gif")
                || filenames[i].toLowerCase().endsWith(".jpg")) {

                // Create and add the thumbnail component.
                // Also need to translate the relative filename to an 
                // absolute filename in order to fetch the image.
                Component c = new Thumbnail(
                    getToolkit().getImage(dir.getAbsolutePath() 
                        + File.separator + filenames[i]), filenames[i]);
                add(c);
                c.setSize(cellSize, cellSize);
            }
        }

        // Listen for events and layout.
        addWindowListener(new WindowEventHandler());
        setLayout(new FlowLayout());
        pack();
        show();
    }

    public Dimension getPreferredSize() {
        Insets insets = getInsets();
        int count = getComponentCount();
        int hgap = ((FlowLayout)getLayout()).getHgap();
        int vgap = ((FlowLayout)getLayout()).getVgap();

        // Maximum of 4 columns,
        int cols = Math.max(1, Math.min(count, 4));

        // The following code exactly determines the size
        // necessary to show all the thumbnails.
        Dimension d = new Dimension(
            cols*(cellSize+hgap) + hgap, 
            ((count-1)/cols+1)*(cellSize+vgap) + vgap);

        // Don't forge the frame's insets.
        d.width += insets.left+insets.right;
        d.height += insets.top+insets.bottom;

        // Make sure it's not larger than the screen.
        Dimension screenDim = getToolkit().getScreenSize();
        d.width = Math.min(d.width, screenDim.width);
        d.height = Math.min(d.height, screenDim.height);
        return d;
    }

    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent evt) {
            // Destroy the window.
            dispose();
        }
    }

    public static void main(String[] args) {
        File dir;

        if (args.length == 1) {
            dir = new File(args[0]);
        } else {
            dir = new File(".");
        }
        new Main(dir);
    }
}

class Thumbnail extends Canvas {
    Image image;
    String name;

    Thumbnail(Image image, String name) {
        this.image = image;
        this.name = name;

        // Start loading the image.
        prepareImage(image, this);

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());
    }

    public void paint(Graphics g) {
        update(g);
    }
    public void update(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height - fm.getHeight();
        int iw = image.getWidth(this);
        int ih = image.getHeight(this);

        if (iw > 0 && ih > 0) {
            // Scale down if necessary.
            if (iw > h) {
                ih = ih * w / iw;
                iw = w;
            }
            if (ih > h && ih > 0) {
                iw = iw * h / ih;
                ih = h;
            }
                
            // Clear background.
            g.setColor(Color.lightGray);
            g.fillRect(0, 0, w, h);

            // Center the image.
            g.drawImage(image, (w-iw)/2, (h-ih)/2, iw, ih, this);
        }
        // Draw the name.
        h = getSize().height;
        g.setColor(Color.black);
        g.clearRect(0, h, w, fm.getHeight());
        g.drawString(name, 
            (w-fm.stringWidth(name))/2, h-fm.getHeight()+fm.getAscent());
        g.drawRect(0, 0, w-1, h-1);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            new ImageViewer(image, name);
        }
        public void mouseEntered(MouseEvent evt) {
            // Set the frame's title.
            findFrame().setTitle(name);
        }
        public void mouseExited(MouseEvent evt) {
            // Clear the frame's title.
            findFrame().setTitle(null);
        }

        // Returns this component's frame.
        public Frame findFrame() {
            Component c = getParent();
            while (c != null && !(c instanceof Frame)) {
                c = c.getParent();
            }
            return (Frame)c;
        }
    }
}

class ImageViewer extends Frame {
    Image image;
    ImageViewer(Image image, String name) {
        super(name);
        this.image = image;

        // Listen for events and show frame.
        addWindowListener(new WindowEventHandler());
        pack();    
        show();
    }

    // Determine the size of the frame that will show the image.
    public Dimension getPreferredSize() {
        Insets insets = getInsets();

        return new Dimension(image.getWidth(null) + insets.left + insets.right,
            image.getHeight(null) + insets.top + insets.bottom);
    }

    public void paint(Graphics g) {
        g.drawImage(image, getInsets().left, getInsets().top, this);
    }

    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent evt) {
            // Destroy the window.
            dispose();
        }
    }
}
