import java.awt.*;
import java.awt.event.*;
import java.beans.*;

public class Map extends Canvas implements ItemSelectable {
    // These fields are used by clients of this class.
    public transient Image image;
    public Area[] areas;
    int curArea = -1;

    // Offscreen image used for double-buffering.
    transient Image bbuf;

    // If true, the map is in a customizer and so the map can be edited.
    boolean inCustomizer = false;

    // If true, the user is dragging an area.
    boolean dragging;

    // If true, the user is stretching an area.
    boolean stretching;

    // Location of cursor when the user pressed the mouse.
    // When dragging, it is relative to the origin of the area rather than
    // to the component.
    // When stretching, it holds distance to the se corner of the area;
    // both coordinates are always positive.
    Point downPt;

    public Map() {
        // Listen for mouse and mouse motion events.
        addMouseMotionListener(new MouseMotionEventHandler());
        addMouseListener(new MouseEventHandler());
    }

    // Change the background image.
    public void setImage(Image newImage) {
        MediaTracker tracker = new MediaTracker(this);
        image = newImage;
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }
        setSize(image.getWidth(null), image.getHeight(null));
        repaint();
    }

    public void setAreas(Area[] newAreas) {
        areas = newAreas;
        curArea = -1;
        repaint();
    }


    // This method is need to implement ItemSelectable.
    public Object[] getSelectedObjects() {
        if (curArea >= 0) {
            Object[] result = {areas[curArea]};

            return result;
        }
        return null;
    }

    // Paint methods.
    public void paint(Graphics g) {
        update(g);
    }

    public void update(Graphics g) {
        // Copy the value in case it gets changed by the event handler methods.
        int ca = curArea;

        if (!inCustomizer || !Beans.isDesignTime()) {
            g.drawImage(image, 0, 0, this);
            return;
        }

        // Create the offscreen image if necessary.
        if (bbuf == null) {
            bbuf = createImage(image.getWidth(null), image.getHeight(null));
        }
        Graphics bbufG = bbuf.getGraphics();

        // Draw the background image.
        bbufG.drawImage(image, 0, 0, this);
        
        // Paint all the area rectangles.
        for (int i=0; i<areas.length; i++) {
            if (i == ca) {
                bbufG.setColor(Color.red);
                // Paint the handle.
                bbufG.fillRect(handleRect.x, handleRect.y, 
                    handleRect.width-1, handleRect.height-1);
            } else {
                bbufG.setColor(Color.black);
            }
            bbufG.drawRect(areas[i].bounds.x, areas[i].bounds.y, 
                areas[i].bounds.width-1, areas[i].bounds.height-1);
        }

        bbufG.dispose();
        g.drawImage(bbuf, 0, 0, this);
    }

    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent evt) {
            if (inCustomizer && Beans.isDesignTime()) {
                return;
            }
            int x = evt.getX();
            int y = evt.getY();

            for (int i=0; i<areas.length; i++) {
                if (areas[i].bounds.contains(x, y)) {
                    showTip(areas[i].label, x, y);
                    return;
                }
            }
            showTip(null, 0, 0);
        }
        public void mouseDragged(MouseEvent evt) {
            if (dragging) {
                areas[curArea].bounds.x = evt.getX()-downPt.x;
                areas[curArea].bounds.y = evt.getY()-downPt.y;

                // Keep the area within the imagemap.
                areas[curArea].bounds.x = Math.max(0, Math.min(
                    areas[curArea].bounds.x, image.getWidth(null)
                    -areas[curArea].bounds.width));
                areas[curArea].bounds.y = Math.max(0, Math.min(
                    areas[curArea].bounds.y, image.getHeight(null)
                    -areas[curArea].bounds.height));
                updateHandle();
                repaint();
            } else if (stretching) {
                // Keep the box within the image.
                int x = Math.min(evt.getX(), image.getWidth(null)-1);
                int y = Math.min(evt.getY(), image.getHeight(null)-1);

                areas[curArea].bounds.width = x+downPt.x
                    -areas[curArea].bounds.x;
                areas[curArea].bounds.height = y+downPt.y
                    -areas[curArea].bounds.y;

                // Don't make the area too small.
                areas[curArea].bounds.width = Math.max(20, 
                    areas[curArea].bounds.width);
                areas[curArea].bounds.height = Math.max(20, 
                    areas[curArea].bounds.height);

                updateHandle();
                repaint();
            }
        }
    }

    void updateHandle() {
        if (curArea >= 0) {
            handleRect = new Rectangle(areas[curArea].bounds);

            handleRect.x = handleRect.x+handleRect.width-5;
            handleRect.y = handleRect.y+handleRect.height-5;
            handleRect.width = 5;
            handleRect.height = 5;
        }
    }

    Rectangle handleRect;
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            int x = evt.getX();
            int y = evt.getY();

            for (int i=0; i<areas.length; i++) {
                if (areas[i].bounds.contains(x, y)) {
                    curArea = i;

                    // Check if on handle
                    updateHandle();
                    if (handleRect.contains(x, y)) {
                        stretching = true;
                        // The down point is relative to the se corner.
                        downPt = new Point(areas[i].bounds.x+
                                areas[i].bounds.width-x, 
                            areas[i].bounds.y+areas[i].bounds.height-y);
                    } else {
                        dragging = true;
                        // The down point is relative to the nw corner.
                        downPt = new Point(x-areas[i].bounds.x, 
                            y-areas[i].bounds.y);
                    }
                    processItemEvent(new ItemEvent(Map.this,
                        ItemEvent.ITEM_STATE_CHANGED, new Integer(i), 
                        ItemEvent.SELECTED));
                    repaint();
                    break;
                }
            }
        }
        public void mouseReleased(MouseEvent evt) {
            dragging = false;
            stretching = false;
        }
    }

    public Dimension getPreferredSize() {
        return new Dimension(image.getWidth(null), image.getHeight(null));
    } 

    // Tip methods
    TipWindow tip;
    public void showTip(String tipLabel, int x, int y) {
        if (tip == null) {
            // Find the parent.
            Component c = getParent();
            while (!(c instanceof Frame)) {
                c = c.getParent();
            }
            tip = new TipWindow((Frame)c);
        }
        tip.setLabel(tipLabel);
        if (tipLabel != null) {
            if (!tip.isShowing()) {
                Point pt = getLocationOnScreen();
    
                tip.setLocation(pt.x+x, pt.y+y);
                tip.show();
            }
        } else {
            tip.setVisible(false);
        }
    }

    // Item listener support methods.
    transient ItemListener itemListener;
    public synchronized void addItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.add(itemListener, l);
    }
    public synchronized void removeItemListener(ItemListener l) {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }
    protected void processItemEvent(ItemEvent evt) {
        if (itemListener != null) {
            itemListener.itemStateChanged(evt);
        }
    }
}
